"""
EuroPMCAnnot_get_annotations_by_type

Get all annotations of a specific type from multiple articles using the Europe PMC Annotations AP...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def EuroPMCAnnot_get_annotations_by_type(
    article_ids: str,
    annotation_type: str,
    page_size: Optional[int | Any] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get all annotations of a specific type from multiple articles using the Europe PMC Annotations AP...

    Parameters
    ----------
    article_ids : str
        Comma-separated article identifiers. Format: 'PMC:PMCXXXXXXX' or 'MED:PMID'. ...
    annotation_type : str
        Annotation type to retrieve. One of: 'Chemicals', 'Organisms', 'Gene Ontology...
    page_size : int | Any
        Number of results per page. Default: 25. Max: 1000.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "article_ids": article_ids,
            "annotation_type": annotation_type,
            "page_size": page_size,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "EuroPMCAnnot_get_annotations_by_type",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["EuroPMCAnnot_get_annotations_by_type"]
