"""
OpenCitations_get_citations

Get the list of papers that cite a given scientific article (its citation count and citing papers...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def OpenCitations_get_citations(
    doi: str,
    limit: Optional[int | Any] = 100,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get the list of papers that cite a given scientific article (its citation count and citing papers...

    Parameters
    ----------
    doi : str
        DOI of the paper to find citations for. Do not include 'https://doi.org/' pre...
    limit : int | Any
        Maximum number of citations to return. Highly-cited papers can have 10,000+ c...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"doi": doi, "limit": limit}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "OpenCitations_get_citations",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["OpenCitations_get_citations"]
