"""
SwissDock_check_job_status

Check the status of a SwissDock docking job using its session ID. Returns current job status: RUN...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def SwissDock_check_job_status(
    session_id: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Check the status of a SwissDock docking job using its session ID. Returns current job status: RUN...

    Parameters
    ----------
    session_id : str
        Session identifier returned from dock_ligand call (format: alphanumeric strin...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"session_id": session_id}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "SwissDock_check_job_status",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["SwissDock_check_job_status"]
