"""
kegg_find_genes

Find genes in KEGG database by keyword. Can search across all organisms or within a specific orga...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def kegg_find_genes(
    keyword: str,
    organism: Optional[str] = "",
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Find genes in KEGG database by keyword. Can search across all organisms or within a specific orga...

    Parameters
    ----------
    keyword : str
        Search keyword for gene names or descriptions
    organism : str
        Organism code (e.g., 'hsa' for human, 'mmu' for mouse). Optional - searches a...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"keyword": keyword, "organism": organism}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "kegg_find_genes",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["kegg_find_genes"]
